# -*- coding: utf-8 -*-
# 12/6/2018

import pandas as pd
import sys
import os
import time
import subprocess
import numpy as np
from baseScripts import *
from INT import * # Inverse rank normalization function (https://github.com/edm1/rank-based-INT/blob/master/rank_based_inverse_normal_transformation.py)
#import matplotlib.pyplot as plt
# module load anaconda/4.3.1
# module load roslin/plink/1.90p

###########################################
# Check user input

cargs = sys.argv[1:]

if len(cargs) == 1:
	CHROM = int(cargs[0])
else:
	print("Wrong input. One value needed:\n")
	print("Chromosome number (1-22).\n")
	sys.exit
print("Selected chromosome: ", CHROM)

###########################################

# MODEL 1: RUN ALL UNRELATED WHITE BRITISH TOGETHER WITH GxS INTERACTION TERM
# MODEL 2: RANK NORMALIZE QUANT. PHENOTYPES WITHIN SEX, AND DO SEX-STRATIFIED GWAS TO LATER COMPARE BETAS

# In this script we are running model 2

###########################################
# Import data

directory = "/exports/igmm/eddie/Tenesa-UKB-Ph1/elena/gender.gwas_european/beta.comparisons/results/sexnorm.gwas_replication/per_trait_linear_scale/"
bfile = "/exports/igmm/eddie/Tenesa-UKB-Ph1/elena/gender.gwas/beta.comparisons_filter/results/PLINK_bedbimfam/chr%s/chr%s_all" % (CHROM, CHROM)
toptraits_df = pd.read_table("/gpfs/igmmfs01/eddie/Tenesa-UKB-Ph1/elena/gender.gwas_european/beta.comparisons/results/exploration/toptraits_T_10-8_table_maf10.tsv", index_col = 0)
toptraits = toptraits_df.index.values.tolist()

fam_file = pd.read_csv(bfile + ".fam", delim_whitespace = True, index_col = 0, header = None)
covfile = "/exports/igmm/eddie/Tenesa-UKB-Ph1/elena/konrad.dump/qc_ukb.sex.allWhites.filtered.miss.noarray_all.cov"
covariates = pd.read_csv(covfile, delim_whitespace = True, index_col = 0, header = None)
print("Loaded fam and cov.")

# Import phenotypes
binary_pheno = "/gpfs/igmmfs01/eddie/Tenesa-UKB-Ph1/ocanela/gwas/allWhites/additionalFiles.allWhites/phenos/binary_e.allWhites.phenos.gz"
quantitative_pheno = "/gpfs/igmmfs01/eddie/Tenesa-UKB-Ph1/ocanela/gwas/allWhites/additionalFiles.allWhites/phenos/quantitative.allWhites.phenos.gz"
binary_df = pd.read_csv(binary_pheno, index_col = 0, delim_whitespace = True)
binary_df = binary_df[list(binary_df)[1:]]
binary_df[binary_df.isin([1])] = 2
binary_df[binary_df.isin([0])] = 1
binary_df = binary_df + np.random.normal(0,0.000001,binary_df.shape)
binary_df = binary_df[binary_df.index.isin(fam_file.index.values.tolist())]
quanti_df = pd.read_csv(quantitative_pheno, index_col = 0, delim_whitespace = True)
quanti_df = quanti_df[list(quanti_df)[1:]]
quanti_df = quanti_df[quanti_df.index.isin(fam_file.index.values.tolist())]
binary_df = binary_df[[trait for trait in toptraits if trait in list(binary_df)]]
quanti_df = quanti_df[[trait for trait in toptraits if trait in list(quanti_df)]]

print("Loaded phenotypes.")

# Normalize quantitative phenotypes within each sex
sex_file = "/gpfs/igmmfs01/eddie/Tenesa-UKB-Ph1/UKB/EGAD00010001226/001/white.qc/genotypes/byGender/metadata/%s.allWhites.dat"
males = [int(line.rstrip().split(" ")[0]) for line in open(sex_file % "males")]
females = [int(line.rstrip().split(" ")[0]) for line in open(sex_file % "females")]

# Rank inverse normalize
print(quanti_df.head())
for trait in list(quanti_df):
	print(trait)
	trait_name = cleanFileName(trait) 

	# Normalize
	quanti_df.loc[quanti_df.index.isin(females), trait] = rank_INT(quanti_df.loc[quanti_df.index.isin(females), trait])
	quanti_df.loc[quanti_df.index.isin(males), trait] = rank_INT(quanti_df.loc[quanti_df.index.isin(males), trait])

print(quanti_df.head())

# Normalize by SD binary traits
sd_df = pd.DataFrame(index = list(binary_df), columns = ["Females", "Males"])
sd_df["Females"] = binary_df[binary_df.index.isin(females)].std(axis = 0)
sd_df["Males"] = binary_df[binary_df.index.isin(males)].std(axis = 0)

# SD normalize each trait
for trait in list(binary_df):
	binary_df.loc[binary_df.index.isin(females), trait] = binary_df.loc[binary_df.index.isin(females), trait]/sd_df.at[trait, "Females"]
	binary_df.loc[binary_df.index.isin(males), trait] = binary_df.loc[binary_df.index.isin(males), trait]/sd_df.at[trait, "Males"]

###########################################
# GWAS

for trait in toptraits:
	print(trait)
	trait_name = cleanFileName(trait) 

	# SNPs to test
	SNPs = pd.read_table("/gpfs/igmmfs01/eddie/Tenesa-UKB-Ph1/elena/gender.gwas_european/beta.comparisons/results/exploration_per.trait/topsnps.LDpruned/T_10-8/topsnps_T.10e8_%s_maf10.tsv" % trait_name, index_col = 0)
	SNPs = SNPs[SNPs["Chromosome"] == CHROM].index.values.tolist()
	if len(SNPs) == 0:
		continue
	snpfile = directory + "snps2use_chr%s_%s_maf10.txt" % (CHROM, trait_name)
	new_file = open(snpfile, "w")
	new_file.write("\n".join(SNPs))
	new_file.close()
	print("Loaded SNPs.") 
	
	# Make trait directory to store GWAS results
	folder = directory + trait_name + "/"
	if not os.path.exists(folder):
	    os.makedirs(folder)

	# Alter fam file with phenotype info for given trait and save
	fam_file_trait = fam_file.copy()
	if trait in list(binary_df):
		fam_file_trait[5] = binary_df[trait]
		fam_file_trait[5] = fam_file_trait[5].fillna(value = -9)
		#fam_file_trait[5] = fam_file_trait[5].apply(np.int64)
	elif trait in list(quanti_df):
		fam_file_trait[5] = quanti_df[trait]
		fam_file_trait[5] = fam_file_trait[5].fillna(value = -9)
	#fam_file_trait = fam_file_trait.fillna(value = 0)
	famfile = folder + "temp_chr%s_%s_maf10.fam" % (CHROM, trait_name)
	fam_file_trait.to_csv(famfile, sep = " ", header = False, index_label = False)
	print(fam_file_trait.head())
	print("Modified fam file.")
	
	# Run PLINK
	print("Running PLINK...")
	outfile_m = folder + "chr%s_males_gxs.GWAS_maf10" % (CHROM)
	outfile_f = folder + "chr%s_females_gxs.GWAS_maf10" % (CHROM)
	# ADD is param #1, followed by all covariates
	# Number of covariates is 149 (if you include ADD 150), range(1, 151) = 1,2,...,150
	# We are NOT interested in sex (cov 149)
	param = list(range(1, 150))
	param = [str(i) for i in param]
	parameters = ",".join(param)

	##### Runk PLINK
	# Females
	os.system("plink --bfile %s --fam %s --linear --keep %s --extract %s --covar %s --allow-no-sex --parameters %s --vif 1000000000000 --ci 0.95 --out %s" % (bfile, famfile, sex_file % "females", snpfile, covfile, parameters, outfile_f))
	# Males
	os.system("plink --bfile %s --fam %s --linear --keep %s --extract %s --covar %s --allow-no-sex --parameters %s --vif 1000000000000 --ci 0.95 --out %s" % (bfile, famfile, sex_file % "males", snpfile, covfile, parameters, outfile_m))

	os.system("rm %s" % famfile)
	os.system("rm %s" % snpfile)

print("Done.")

